# -*- coding: utf-8 -*-

from numpy import matrix
from PyQt4.QtGui import QPen, QBrush, QColor

class Shape(object):
	def __init__(self, name, color):
		self.name = name
		self.color = color

	def __eq__(self, other):
		return (self.coords == other.coords)

	def __ne__(self, other):
		return not (self == other)

	def __str__(self):
		return self.name + str(self.coords)
		
	def __repr__(self):
		return str(self)
	
	@property
	def pen(self):
		return QPen( QColor(*self.color) )

class Point(Shape):
	def __init__(self, name, coordPair, color = (0,0,0)):
		Shape.__init__(self, name, color)
		self.wc = self.ppc = coordPair

	@property
	def center(self):
		return self.wc

	@property
	def centerPPC(self):
		return self.ppc

	@property
	def coords(self):
		return [self.wc]

	@property
	def ppcCoords(self):
		return [self.ppc]

	def applyTransform(self, t):
		self.wc = t(self.wc)

	def applyToPPC(self, t):
		self.ppc = t(self.ppc)

class Line(Shape):
	def __init__(self, name, pairA, pairB, color = (0,0,0)):
		Shape.__init__(self, name, color)
		self.wca = self.ppca = pairA
		self.wcb = self.ppcb = pairB

	@property
	def center(self):
		xm = (self.wca[0] + self.wcb[0]) / 2.0
		ym = (self.wca[1] + self.wcb[1]) / 2.0
		return (xm, ym)

	@property
	def centerPPC(self):
		xm = (self.ppca[0] + self.ppcb[0]) / 2.0
		ym = (self.ppca[1] + self.ppcb[1]) / 2.0
		return (xm, ym)

	@property
	def coords(self):
		return [self.wca,  self.wcb]

	@property
	def ppcCoords(self):
		return [self.ppca, self.ppcb]

	def applyTransform(self, t):
		self.wca, self.wcb = t(self.wca), t(self.wcb)

	def applyToPPC(self, t):
		self.ppca, self.ppcb = t(self.ppca), t(self.ppcb)


class Polygon(Shape):
	def __init__(self, name, coordsList, color = (0,0,0)):
		Shape.__init__(self, name, color)
		self.wc = self.ppc = coordsList

	@property
	def center(self):
		numPoints = float(len(self.wc))
		xSum = sum([ c[0] for c in self.wc ])
		ySum = sum([ c[1] for c in self.wc ])
		return (xSum/numPoints, ySum/numPoints)

	@property
	def centerPPC(self):
		numPoints = float(len(self.ppc))
		xSum = sum([ c[0] for c in self.ppc ])
		ySum = sum([ c[1] for c in self.ppc ])
		return (xSum/numPoints, ySum/numPoints)

	@property
	def coords(self):
		return self.wc + [self.wc[0]]

	@property
	def ppcCoords(self):
		return self.ppc + [self.ppc[0]]

	def applyTransform(self, t):
		self.wc = map(t, self.wc)

	def applyToPPC(self, t):
		self.ppc = map(t, self.ppc)
	
class Curve(Shape):
	def __init__(self, name, coords, color = (0,0,0)):
		Shape.__init__(self, name, color)
		
		self._matrix = matrix([[-1, 3, -3, 1], [3, -6, 3, 0], [-3, 3, 0, 0], [1, 0, 0, 0]])
		self._x = matrix([p[0] for p in coords]).T # transposta dos x
		self._y = matrix([p[1] for p in coords]).T # transposta dos y
		
		self._points = coords
		self.wc = self.generateCoords()
		self.ppc = self.wc[:] # clonando lista

	@property
	def center(self):
		xSum = self._x.sum()
		ySum = self._y.sum()
		return (xSum/4.0, ySum/4.0)

	@property
	def centerPPC(self):
		xSum = self._x.sum()
		ySum = self._y.sum()
		return (xSum/4.0, ySum/4.0)

	@property
	def ppcCoords(self):
		return self.ppc

	def applyTransform(self, t):
		self._points = map(t, self._points)
		# TODO considerar estado da window
		self.wc = self.generateCoords()
		self.ppc = self.wc[:]

	def applyToPPC(self, t):
		points = map(t, zip(self._x.A1.tolist(),  self._y.A1.tolist()))
		self._x = matrix([p[0] for p in points]).T
		self._y = matrix([p[1] for p in points]).T
		self.ppc = self.generateCoords()

	def generateCoords(self,  step = 0.01):
		coords = []
		t = 0
		while t <= 1:
			t2 = t*t
			t3 = t2*t
			tmh  = matrix([t3, t2, t, 1]) * self._matrix
			x = (tmh * self._x)[0, 0]
			y = (tmh * self._y)[0, 0]
			t += step
			coords.append((x,y))
		return coords
