# -*- coding: utf-8 -*-
from PyQt4.QtCore import QPointF
from PyQt4.QtGui import QGraphicsScene, QPen, QBrush, QColor, QPolygonF

from model.shapes import Polygon, Point, Line, Curve
from transform.clipper import clipLine
from transform.transforms import viewportTransform, translate, scale, canvas_size

class Window(object):
	def __init__(self, canvas, displayFile):
		self.canvas = canvas
		self.displayFile = displayFile
		
		self.scene = QGraphicsScene(0,0,canvas_size,canvas_size)
		self.canvas.setScene(self.scene)
		
		self.drawDict = {Point:self.drawPoint,Line:self.drawLine,Polygon:self.drawPolygon,Curve:self.drawCurve}
		
		self.vupAngle = 0
		hc = canvas_size/2
		self.winShape = Polygon("window", [(-hc,-hc), (-hc,hc), (hc,hc), (hc,-hc)] )

	@property
	def winPPCmin(self):
		return self.winShape.ppcCoords[0]

	@property
	def winPPCmax(self):
		return self.winShape.ppcCoords[2]

	def drawPoint(self, s):
		x, y = viewportTransform(s.ppcCoords[0], self.winPPCmin, self.winPPCmax)
		self.scene.addLine(x-1, y, x+1, y, s.pen)

	def drawLine(self, s):
		begin, end = s.ppcCoords
		self.drawSegment(s.pen, begin, end)
			
	def drawSegment(self, pen, p1, p2):
		clipped = clipLine(p1, p2, self.winPPCmin, self.winPPCmax)
		if clipped != None:
			(tp0, tp1) = [viewportTransform(p, self.winPPCmin, self.winPPCmax) for p in clipped]
			self.scene.addLine(tp0[0], tp0[1], tp1[0], tp1[1], pen)
	
	def drawPolygon(self, s):
		ppc = s.ppcCoords
		lineSegs = zip( ppc, ppc[1:]+ppc[:1] )
		for l in lineSegs:
			self.drawSegment(s.pen, *l)

	def drawCurve(self, s):
		ppc = s.ppcCoords
		detailLevel = 10.0
		apr, aprStep = s.generateCoords( 1.0/detailLevel ),  len(ppc)/detailLevel
		aprLines = zip(apr[:-1],  apr[1:])
		tp = []
		segs = []
		
		for i, line in enumerate(aprLines):
			curAprox = ppc[ int(i*aprStep) : int((i+1)*aprStep)+1 ]
			c = clipLine(line[0],  line[1],  self.winPPCmin,  self.winPPCmax)
			if c is None: continue
			
			if c == line: 
				tp += [ viewportTransform(p,self.winPPCmin,self.winPPCmax) for p in curAprox ]
			else:
				for (p1,p2) in zip(curAprox[:-1], curAprox[1:]):
					c = clipLine(p1, p2, self.winPPCmin, self.winPPCmax)
					if c is not None:
						vp = tuple([viewportTransform(p,self.winPPCmin,self.winPPCmax) for p in c])
						segs.append(vp)
		
		segs += zip(tp[:-1], tp[1:])
		for (p1,p2) in segs:
			self.scene.addLine(p1[0], p1[1], p2[0], p2[1], s.pen)


	def redraw(self):
		self.scene.clear()
		for s in self.displayFile.shapes:
			self.drawDict[s.__class__](s)
	
	def zoom(self, ratio):
		scale(self.winShape, (ratio,ratio) )
		self.redraw()

	def move(self, moveVector):
		translate(self.winShape, moveVector)
		self.redraw()

	def rotate(self, angle):
		self.vupAngle = (self.vupAngle + angle) % 360
		self.displayFile.rotateAllObjectsPPC(-angle, (0,0) )
